@@ -7,7 +7,7 @@
 ******************************************************************************/
 
 /*
- * $Id: riffwave.c,v 1.3 2002/01/27 15:37:21 jason Exp $
+ * $Id: riffwave.c,v 1.5 2002/11/16 18:03:07 jason Exp $
  */
 
 #include <stdio.h>
@@ -31,18 +31,6 @@
 #define WAVE_FORMAT_ALAW      0x0006
 #define WAVE_FORMAT_MULAW     0x0007
 
-/*
- * This structure contains an array of `char *' pointers to blocks
- * of data to be coded verbatim. The structure itself will be
- * dynamically allocated, as will the array of pointers, as will
- * the blocks themselves.
- */
-struct Riff_Wave_Header {
-  int nblocks, nblk_alloc;
-  uchar **blocks;
-  unsigned *blklen;
-};
-
 /*
  * Portably read a little-endian long integer from a file.
  */
@@ -66,8 +54,8 @@ static sshort short_at (p) uchar *p; {
 /*
  * Write a load of data into a header chunk.
  */
-static void write_hdr(buf, len, hdr)
-uchar *buf; int len; Riff_Wave_Header *hdr;
+void write_hdr(buf, len, hdr)
+const uchar *buf; int len; Iff_Header *hdr;
 {
   while (len--) {
     if (hdr->nblocks == 0 ||
@@ -102,7 +90,7 @@ uchar *buf; int len; Riff_Wave_Header *hdr;
  * Write a little-endian long integer into a header chunk.
  */
 static void write_hdr_long(data, hdr)
-ulong data; Riff_Wave_Header *hdr;
+ulong data; Iff_Header *hdr;
 {
   uchar realdata[4];
   realdata[0] = data & 0xFF;
@@ -112,10 +100,10 @@ ulong data; Riff_Wave_Header *hdr;
   write_hdr (realdata, 4, hdr);
 }
 
-Riff_Wave_Header *riff_wave_prochdr (filei, ftype, nchan, datalen, wtype)
+Iff_Header *riff_wave_prochdr (filei, ftype, nchan, datalen, wtype)
 FILE *filei; int *ftype, *nchan; slong *datalen; int *wtype;
 {
-  Riff_Wave_Header *hdr;
+  Iff_Header *hdr;
   slong len, ckhdr;
   int seen_fmt = 0;
 
@@ -130,8 +118,13 @@ FILE *filei; int *ftype, *nchan; slong *datalen; int *wtype;
   hdr->blklen = NULL;
 
   /* Read the RIFF signature. */
-  if (fread_long(filei) != RIFF_SIGNATURE ||
-      (len = fread_long(filei)) < 12 ||
+  /* NOTE: This comparison has been moved out to shorten.c to support AIFF
+  if (fread_long(filei) != RIFF_SIGNATURE) {
+    free_header(hdr);
+    return NULL;
+  }
+  */
+  if ((len = fread_long(filei)) < 12 ||
       fread_long(filei) != WAVE_SIGNATURE) {
     free_header(hdr);
     return NULL;
@@ -226,7 +219,7 @@ uchar *data; int len; FILE *fileo;
 }
 
 void write_header (hdr, fileo)
-Riff_Wave_Header *hdr; FILE *fileo;
+Iff_Header *hdr; FILE *fileo;
 {
   uchar **blocks;
   unsigned *blklen;
@@ -250,7 +243,7 @@ FILE *filei, *fileo;
 }
 
 void free_header (hdr)
-Riff_Wave_Header *hdr;
+Iff_Header *hdr;
 {
   uchar **blocks;
   int nblocks;
