@@ -7,7 +7,7 @@
 ******************************************************************************/
 
 /*
- * $Id: shorten.c,v 1.22 2003/01/11 01:28:15 jason Exp $
+ * $Id: shorten.c,v 1.27 2004/05/03 06:59:02 jason Exp $
  */
 
 #ifdef _WINDOWS
@@ -92,6 +92,7 @@ int ReadingFunctionCode=FALSE;
 int WriteSeekTable=TRUE;
 int WriteWaveFile=FALSE;
 int AppendSeekInfo=FALSE;
+int DupFileInfo=FALSE;
 
 slong CBuf_0_Minus1 = 0;
 slong CBuf_0_Minus2 = 0;
@@ -312,21 +313,76 @@ void ushort_to_uchar_le(uchar * buf,ushort num)
   buf[1] = tmp[1];
 }
 
-BOOL FileContainsSeekInfo(FILE *filei)
+BOOL FileHasSeekInfoAppended(FILE *filei,ulong *seektable_version)
 {
-  char SeekSignature[9]="SHNAMPSK";
-  char Buffer[9]="        ";
+  char SeekTrailerSignature[9]="SHNAMPSK";
+  char SeekHeaderSignature[5]="SEEK";
+  TSeekTableTrailer SeekTrailer;
+  TSeekTableHeader SeekHeader;
 
   long InitialPosition=ftell(filei);
 
-  fseek(filei,-8,SEEK_END);
-  fread(Buffer,1,8,filei);
-  fseek(filei,InitialPosition,SEEK_SET);
+  fseek(filei,-SEEK_TRAILER_SIZE,SEEK_END);
+  fread(SeekTrailer.data,1,SEEK_TRAILER_SIZE,filei);
 
-  if(0==memcmp(SeekSignature,Buffer,8))
+  if(0==memcmp(SeekTrailerSignature,SeekTrailer.data+4,8)) {
+    if (seektable_version) {
+      fseek(filei,-uchar_to_ulong_le(SeekTrailer.data),SEEK_END);
+      fread(SeekHeader.data,1,SEEK_HEADER_SIZE,filei);
+
+      if (0==memcmp(SeekHeaderSignature,SeekHeader.data,4)) {
+        *seektable_version = uchar_to_ulong_le(SeekHeader.data+4);
+      }
+      else {
+        *seektable_version = -1;
+      }
+    }
+
+    fseek(filei,InitialPosition,SEEK_SET);
     return TRUE;
-  else
-    return FALSE;
+  }
+
+  fseek(filei,InitialPosition,SEEK_SET);
+  return FALSE;
+}
+
+BOOL FileIsExternalSeekTable(FILE *filei,ulong *seektable_version)
+{
+  char SeekHeaderSignature[5]="SEEK";
+  TSeekTableHeader SeekHeader;
+
+  long InitialPosition=ftell(filei);
+
+  fread(SeekHeader.data,1,SEEK_HEADER_SIZE,filei);
+
+  if (0==memcmp(SeekHeaderSignature,SeekHeader.data,4)) {
+    if (seektable_version) {
+      *seektable_version = uchar_to_ulong_le(SeekHeader.data+4);
+    }
+
+    fseek(filei,InitialPosition,SEEK_SET);
+    return TRUE;
+  }
+
+  fseek(filei,InitialPosition,SEEK_SET);
+  return FALSE;
+}
+
+void ShowSeekInfo(FILE *f,char *filename)
+{
+  ulong seektable_version;
+
+  if (FileIsExternalSeekTable(f,&seektable_version)) {
+    fprintf(stderr,"File '%s' is an external seek table file (revision %lu).\n",filename,(unsigned long)seektable_version);
+    return;
+  }
+
+  if (FileHasSeekInfoAppended(f,&seektable_version)) {
+    fprintf(stderr,"File '%s' contains appended seek tables (revision %lu).\n",filename,(unsigned long)seektable_version);
+    return;
+  }
+
+  fprintf(stderr,"File '%s' does not contain seek information.\n",filename);
 }
 
 void RemoveSeekInfo(char *filename)
@@ -406,7 +462,7 @@ int shorten(FILE *stdi, FILE *stdo,int argc,char **argv)
   int   version = MAX_SUPPORTED_VERSION, extract = 0, lastbitshift = 0, bitshift = 0, want_seeking = 1;
   int   hiloint = 1, hilo = !(*((char*) &hiloint));
   int   ftype = TYPE_EOF;
-  char  *magic = MAGIC, *filenamei = NULL, *filenamei_orig = NULL;
+  char  *magic = MAGIC, *filenamei = NULL, *filenamei_orig = NULL, *dupfilenamei = NULL, *dupfilenameo = NULL;
   char  *tmpfilename = NULL;
   char  *maxresnstr = DEFAULT_MAXRESNSTR;
   FILE  *filei = 0;
@@ -461,7 +517,7 @@ int shorten(FILE *stdi, FILE *stdo,int argc,char **argv)
   SeekTableHeader.data[1]='E';
   SeekTableHeader.data[2]='E';
   SeekTableHeader.data[3]='K';
-  ulong_to_uchar_le(SeekTableHeader.data+4,0);
+  ulong_to_uchar_le(SeekTableHeader.data+4,SEEK_TABLE_REVISION);
   ulong_to_uchar_le(SeekTableHeader.data+8,0);
 
   SeekTableTrailer.data[4]='S';
@@ -794,10 +850,11 @@ StartAgain:
       usage_exit(1, "could not open file '%s' for input\n", filenamei);
 
     if (saw_e_op || saw_i_op) {
-      if (saw_i_op)
-        fprintf(stderr,"file '%s' %s seek information.\n",filenamei,FileContainsSeekInfo(filei)?"contains":"does not contain");
+      if (saw_i_op) {
+        ShowSeekInfo(filei,filenamei);
+      }
       else {
-        if (!FileContainsSeekInfo(filei))
+        if (!FileHasSeekInfoAppended(filei,NULL))
           usage_exit(1,"file '%s' does not contain seek information\n",filenamei);
         RemoveSeekInfo(filenamei);
       }
@@ -833,7 +890,7 @@ StartAgain:
   {
     if(WriteSeekTable && extract && AppendSeekInfo)
     {
-      if(FileContainsSeekInfo(filei))
+      if(FileHasSeekInfoAppended(filei,NULL))
         usage_exit(1, "file '%s' already contains seek information\n", filenamei);
     }
 
@@ -1323,6 +1380,7 @@ StartAgain:
     int i, cmd;
     int internal_ftype;
 
+    bitshift = 0;
     bytes_read = 0;
 
     /* Firstly skip the number of bytes requested in the command line */
@@ -1701,9 +1759,12 @@ StartAgain:
   if(fileo && fileo != stdo)
     fclose(fileo);
 
-  /* make the output file look like the original if possible */
-  if((filei != stdi) && (fileo != stdo))
-    (void) dupfileinfo(filenamei, filenameo);
+  /* make the output file look like the original if possible - delay this until after seek tables are created (if any) */
+  if((filei != stdi) && (fileo != stdo) && (0 == Pass)) {
+    DupFileInfo = TRUE;
+    dupfilenamei = filenamei;
+    dupfilenameo = filenameo;
+  }
 
   filei = fileo = 0;
 
@@ -1802,6 +1863,9 @@ StartAgain:
     fclose(SeekTableFile);
   }
 
+  if (DupFileInfo)
+    (void) dupfileinfo(dupfilenamei, dupfilenameo);
+
   if(tmpfilename != NULL)
     free(tmpfilename);
 
